package cn.com.qmth.stmms.common.authorization;

import cn.com.qmth.stmms.common.domain.AuthInfo;
import cn.com.qmth.stmms.common.signature.SignatureInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;

@Service("authorizationService")
public class AuthorizationService {

    private static final String HEADER_AUTHORIZATION_KEY = "authorization";

    private static final String HEADER_DATE_KEY = "date";

    private static final String HEADER_AUTHINFO_KEY = "auth-info";

    private static final long TIME_AHEAD_SECOND = 5;

    private static final long TIME_EXPIRE_SECOND = 15;

    public SignatureInfo buildSignature(HttpServletRequest request) {
        long timestamp = 0;
        try {
            timestamp = Long.parseLong(StringUtils.trimToNull(request.getHeader(HEADER_DATE_KEY)));
        } catch (Exception e) {
        }
        if (!expired(timestamp)) {
            return SignatureInfo.parse(request.getMethod(), request.getServletPath(), timestamp,
                    StringUtils.trimToNull(request.getHeader(HEADER_AUTHORIZATION_KEY)));
        }
        return null;
    }

    public AuthInfo buildAuthInfo(HttpServletRequest request) {
        return AuthInfo.parse(StringUtils.trimToEmpty(request.getHeader(HEADER_AUTHINFO_KEY)));
    }

    private boolean expired(long timestamp) {
        long diff = (System.currentTimeMillis() - timestamp) / 1000;
        return diff < (-1 * TIME_AHEAD_SECOND) || diff > TIME_EXPIRE_SECOND;
    }

}
