package cn.com.qmth.stmms.common.domain;

import cn.com.qmth.stmms.biz.school.model.School;
import cn.com.qmth.stmms.biz.user.model.OpenAccount;
import org.apache.commons.lang.StringUtils;

import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.session.model.StmmsSession;

/**
 * web端访问用户抽象描述类
 *
 * @author luoshi
 */
public class WebUser {

    private Integer id;

    private String name;

    private Role role;

    private Object userData;

    private String token;

    private String logoutUrl;

    private WebUser(Integer id, String name, Role role, String token, Object userData, String logoutUrl) {
        this.id = id;
        this.name = name;
        this.role = role;
        this.token = token;
        this.userData = userData;
        this.logoutUrl = logoutUrl;
    }

    public WebUser(User user) {
        this(user.getId(), user.getName(), user.getRole(), user.getAccessToken(), user, null);
    }

    public WebUser(Marker marker, String logoutUrl) {
        this(marker.getId(), marker.getName(), Role.MARKER, marker.getAccessToken(), marker, logoutUrl);
    }

    public WebUser(Marker marker) {
        this(marker, null);
    }

    public User getUser() {
        return userData != null && userData instanceof User ? (User) userData : null;
    }

    public Marker getMarker() {
        return userData != null && userData instanceof Marker ? (Marker) userData : null;
    }

    public Integer getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public Role getRole() {
        return role;
    }

    public String getToken() {
        return token;
    }

    public String getLogoutUrl() {
        return logoutUrl;
    }

    public boolean isSubjectHeader() {
        return role == Role.SUBJECT_HEADER;
    }

    public boolean isSchoolAdmin() {
        return role == Role.SCHOOL_ADMIN;
    }

    public boolean isSchoolViewer() {
        return role == Role.SCHOOL_VIEWER;
    }

}
