package cn.com.qmth.stmms.mark.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.session.model.StmmsSession;
import cn.com.qmth.stmms.common.utils.AccessControlUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.interceptor.SessionInterceptor;
import cn.com.qmth.stmms.common.utils.RequestUtils;

/**
 * Mark模块操作拦截器
 *
 * @author XKJ
 */
public class MarkInterceptor extends SessionInterceptor {

    @Autowired
    private MarkerService markerService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        WebUser wu = buildWebUser(request);
        if (wu != null) {
            if (!request.getServletPath().endsWith("mark/reset") && wu.getMarker().getLastLoginTime() == null) {
                // 首次登录，强制重置密码及姓名
                return redirect(request, response, "/mark/reset");
            }
            return true;
        }
        return sessionExpire(request, response, "/login");
    }

    private WebUser buildWebUser(HttpServletRequest request) {
        StmmsSession session = RequestUtils.getSession(request);
        Integer userId = session.getWebUserId();
        Role role = session.getWebUserRole();
        if (userId != null && Role.MARKER == role) {
            Marker marker = markerService.findById(userId);
            if (marker != null && marker.getAccessTokenRefreshTime() != null && StringUtils
                    .equals(session.getWebUserToken(), marker.getAccessToken()) && !AccessControlUtils
                    .expired(marker.getAccessTokenRefreshTime())) {
                WebUser wu = new WebUser(marker, session.getWebUserLogoutUrl());
                RequestUtils.setWebUser(request, wu);
                return wu;
            }
        }
        return null;
    }
}
