package cn.com.qmth.stmms.open.controller;

import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.school.model.School;
import cn.com.qmth.stmms.biz.school.service.SchoolService;
import cn.com.qmth.stmms.biz.user.model.OpenAccount;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.authorization.AuthorizationService;
import cn.com.qmth.stmms.common.domain.OpenUser;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.AccountType;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import cn.com.qmth.stmms.common.session.model.StmmsSession;
import cn.com.qmth.stmms.common.signature.SignatureInfo;
import cn.com.qmth.stmms.common.signature.SignatureType;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

@Controller("openAccountController")
@RequestMapping("/open")
public class OpenAccountController {

    @Autowired
    private AuthorizationService authorizationService;

    @Autowired
    private UserService userService;

    @Autowired
    private SchoolService schoolService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private MarkGroupService groupService;

    @RequestMapping(value = "/marker/login", method = RequestMethod.POST)
    public String markerLogin(HttpServletRequest request, RedirectAttributes redirect, @RequestParam String account,
            @RequestParam String returnUrl) {
        // 请求参数验证
        account = StringUtils.trimToEmpty(account);
        returnUrl = StringUtils.trimToEmpty(returnUrl);
        if (returnUrl.length() == 0) {
            return errorRedirect(redirect, null, "returnUrl不能为空");
        }
        if (account.length() == 0) {
            return errorRedirect(redirect, returnUrl, "account不能为空");
        }
        // 签名验证，获取访问学校
        School school = null;
        SignatureInfo info = authorizationService.buildSignature(request);
        if (info != null && info.getType() == SignatureType.SECRET) {
            school = schoolService.findByAccessKey(info.getInvoker());
        }
        if (school == null || !school.isEnable() || !info.validate(school.getAccessSecret())) {
            return errorRedirect(redirect, returnUrl, "接口鉴权失败");
        }
        // 查找第三方账号并登陆
        OpenAccount oa = userService.openAccountLogin(school.getId(), account);
        if (oa == null || !oa.isEnable()) {
            return errorRedirect(redirect, returnUrl, "第三方账号不存在");
        }
        // 保存第三方账号到session
        RequestUtils.getSession(request).saveOpenUser(new OpenUser(oa, returnUrl));
        return "redirect:/open/marker/list";
    }

    @RequestMapping(value = "/marker/list", method = RequestMethod.GET)
    public ModelAndView markerList(HttpServletRequest request) {
        OpenAccount account = RequestUtils.getOpenUser(request).getOpenAccount();
        ModelAndView modelAndView = new ModelAndView("modules/open/marker-list");
        // TODO - 增加评卷员列表，构造考试、科目、分组联动数据集合
        List<Marker> markerList = markerService
                .findByAccountAndMarkStatus(AccountType.OPEN_ACCOUNT, account.getId(), MarkStatus.TRIAL, MarkStatus.FORMAL);
        return modelAndView;
    }

    @RequestMapping(value = "/marker/select", method = RequestMethod.POST)
    public ModelAndView markerSelect(HttpServletRequest request, @RequestParam Integer markerId) {
        // 评卷员与评卷分组校验
        OpenAccount account = RequestUtils.getOpenUser(request).getOpenAccount();
        Marker marker = markerService.findById(markerId);
        if (marker == null || !marker.isEnable() || AccountType.OPEN_ACCOUNT != marker.getAccountType() || marker.getAccountId() == null
                || !marker.getAccountId().equals(account.getId())) {
            return new ModelAndView("redirect:/open/marker/list");
        }
        MarkGroup group = groupService.findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        if (group == null || group.getStatus() == MarkStatus.FINISH) {
            return new ModelAndView("redirect:/open/marker/list");
        }
        // 完成评卷员登陆并保存session
        marker.refreshAccessToken();
        marker.setLastLoginIp(request.getRemoteAddr());
        marker.setLastLoginTime(new Date());
        RequestUtils.getSession(request).saveWebUser(new WebUser(markerService.save(marker), "/open/marker/list"));
        return new ModelAndView("redirect:/mark/index");
    }

    @RequestMapping(value = "/error")
    public ModelAndView error(HttpServletRequest request, @RequestParam(required = false) String message,
            @RequestParam(required = false) String returnUrl) {
        ModelAndView modelAndView = new ModelAndView("modules/open/error");
        modelAndView.addObject("message", StringUtils.trimToNull(message));
        modelAndView.addObject("returnUrl", StringUtils.trimToNull(returnUrl));
        return modelAndView;
    }

    private String errorRedirect(RedirectAttributes redirect, String returnUrl, String... errors) {
        if (errors != null && errors.length > 0) {
            redirect.addFlashAttribute("message", StringUtils.join(errors, "\n"));
        }
        if (returnUrl != null && returnUrl.length() > 0) {
            redirect.addFlashAttribute("returnUrl", returnUrl);
        }
        return "redirect:/open/error";
    }
}
